function script_demo_penguin ()
%
%   Demo code.
%
% -------
% INPUTS:
%
% --------
% OUTPUTS:
%
% -------------
% Written by Gunhee Kim @ CMU, 2012.
% All rights reserved.
%

close all ;
warning off ;

% % --------------------------------------------------------------%
%     0. PARAMETERS
% % --------------------------------------------------------------%

% learning mode
% You can choose one of {'glmfit', 'lassoglm'} ;
% glmfit: Poisson GLM fitting
% lassoglm : Poisson GLM fitting with L-1 (or Lasso) penalty
LEARNING_MODE = 'glmfit' ;

% order of Auto Regression
ARO = 6 ;

% load the input file
%   VC              : visual cluster info
%   .desciption     : description of VC
%   .y_obs          : observed VC occurrence data
%   .img_example    : example images.
%   time_tick       : time tick (month)
load ('input_vc_penguin.mat', 'VC', 'time_tick') ;

% % --------------------------------------------------------------%
%     1. Get Covariate matrix for learning
% % --------------------------------------------------------------%

% time tick
bins.tick = time_tick ;
nTick = length(bins.tick) ;

% number of visual clusters (VC)
nVC = length(VC) ;

% [nVC x nTick x ARO]
cov_mat_hist = zeros(nVC, nTick, ARO) ;
for v=1:nVC,
    for i=1:nTick,
        if VC(v).y_obs(i) <= 0, continue ; end
        for k=1:ARO,
            locind = i + k ;
            if locind <= nTick,
                cov_mat_hist(v,locind,k) = cov_mat_hist(v,locind,k) + VC(v).y_obs(i) ;
            end
        end
    end
end

t2 = toc ;
fprintf('getting covariates is done in %f sec.\n', t2) ;


% % --------------------------------------------------------------%
%     2. Learning for history component
% % --------------------------------------------------------------%

tic
% initialization
% tModHist : specific models
%   .id : covariate type
%   .betaSet : beta Set
%   .nll : negative log-likelihood.
tModHist = [] ;
tModHist.betaSet = zeros(nVC, 1+ARO) ;
tModHist.nll = zeros(nVC, 1) ;

for v=1:nVC,

    % get covariates
    vCov = reshape(cov_mat_hist(v,:,:), [nTick, ARO]) ;

    % learning
    [b nll] = learn_poisson_regression (full(VC(v).y_obs)', ...
        vCov, LEARNING_MODE) ;

    tModHist(1).betaSet(v,:) = b ;
    tModHist(1).nll(v) = nll ;

    % compute intensity function
    cif = exp(b(1)+eye(length(b)-1)*b(2:end));

    % (DEBUG)
    % figure; plot(1:length(cif),cif)
end

t2 = toc ;
fprintf('learning for history is done in %f sec.\n', t2) ;


% % --------------------------------------------------------------%
%     3. Learning for correlation component
% % --------------------------------------------------------------%

tic
% initialization
% tModCorr : specific models
%   .id : covariate type
%   .betaSet : beta Set
%   .nll : negative log-likelihood.
tModCorr = [] ;
tModCorr.betaSet = zeros(nVC, nVC, 1+ARO) ;
tModCorr.nll = zeros(nVC, nVC) ;


for v=1:nVC,
    for w=1:nVC,
        if v==w, continue; end

        % get covariates
        vCov = reshape(cov_mat_hist(w,:,1:ARO), [nTick, ARO]) ;

        % learning
        [b nll] = learn_poisson_regression (full(VC(v).y_obs)', ...
            vCov, LEARNING_MODE) ;

        tModCorr(1).betaSet(v, w, :) = b ;
        tModCorr(1).nll(v, w) = nll ;

        % compute intensity function
        % cif = exp(b(1)+eye(length(b)-1)*b(2:end));
        % (DEBUG)
        % figure; plot(1:length(cif),cif)
    end
end

t2 = toc ;
fprintf('learning for correlation is done in %f sec.\n', t2) ;


% % --------------------------------------------------------------%
%     3. Drawing the results
% % --------------------------------------------------------------%

hf = figure ;

% (1) draw example images of each visual cluster
for v=1:nVC,
    imgVC = draw_imgs_big_canvas(VC(v).img_example) ;
    subplot(4, nVC, v) ; imshow(imgVC) ;
    title(VC(v).desciption) ;
end


% (2) draw observed VC occurrences
xlimR = find(sum(vertcat(VC.y_obs))>3) ;
for v=1:nVC,
    hv = subplot(4, nVC, nVC + v) ;
    hb = bar(1:nTick, full(VC(v).y_obs), 'group', 'BarWidth', 1.0) ;
    set(hb,'facecolor',[1 0 0]) % or use RGB triple
    xlim([min(xlimR)-1 max(xlimR)+1]) ;
    title('observed VC occurrences') ;
end


% (3) draw parameters for history
for v=1:nVC,
    hv = subplot(4, nVC, 2*nVC + v) ;
    hb = bar(1:ARO, tModHist.betaSet(v,2:end), 'group', 'BarWidth', 1.0) ;
    set(hb,'facecolor',[0 0 1]) % or use RGB triple
    xlim([0 ARO+1]) ;
    title('parameters for history') ;
end


% (4) draw parameters for correlation
for v=1:nVC,
    hv = subplot(4, nVC, 3*nVC + v) ;
    vBeta = reshape(tModCorr.betaSet(v,:,:), [nVC, ARO+1])' ;
    vBeta(1,:) = 0 ;
    hb = bar(1:nVC*(ARO+1), vBeta(:), 'group', 'BarWidth', 1.0) ;
    set(hb,'facecolor',[0 1 0]) % or use RGB triple
    xlim([1 nVC*(ARO+1)]) ;
    cInd = 1:nVC ; cInd(v) = [] ;
    title([sprintf('parameters for correlation \n with VC ') num2str(cInd)]) ;
end

disp('Done') ;


end

