function script_demo_shark ()
%
%   Demo code.
%   It will generate a figure similar to Fig.3 of [1].
%
% -------
% INPUTS:
%
% --------
% OUTPUTS:
%
% -------------
% Written by Gunhee Kim @ CMU, 2012.
% All rights reserved.
%

close all ;
warning off ;

% % --------------------------------------------------------------%
%     0. PARAMETERS
% % --------------------------------------------------------------%

% learning mode
% You can choose one of {'glmfit', 'lassoglm'} ;
% glmfit: Poisson GLM fitting
% lassoglm : Poisson GLM fitting with L-1 (or Lasso) penalty
LEARNING_MODE = 'glmfit' ;

% COVARIATES: temporal attributes
% y: year, m: month
COVARIATES_ATTRIB   = 'ym' ;

% When computing time covariates, using impulse (0) or Gaussian function (1) ?
COVARIATES_WIND_GAUSS = 0 ;

% load the input file
%   VC              : visual cluster info
%   .desciption     : description of VC
%   .y_obs          : observed VC occurrence data
%   .img_example    : example images.
%   time_tick       : time tick (day)
load ('input_vc_shark.mat', 'VC', 'time_tick') ;

% % --------------------------------------------------------------%
%     1. Get Covariate matrix for learning
% % --------------------------------------------------------------%

bins.tick = time_tick ;
nTick = length(bins.tick) ;

covSet = [] ;
tic ;
% get year covariates
covSet{1} = get_mtr_pp_covari_attrib_year(bins, COVARIATES_ATTRIB) ;
% get month covariates
covSet{2} = get_mtr_pp_covari_attrib_month(bins, COVARIATES_ATTRIB, COVARIATES_WIND_GAUSS) ;

% dimension of each covariate
nCovSet = length(covSet) ;
dimCovSet = zeros(1, nCovSet) ;
for i=1:nCovSet,
    dimCovSet(i) = size(covSet{i}, 2) ;
end
covType = find(dimCovSet>0) ;

% number of visual clusters (VC)
nVC = length(VC) ;

t2 = toc ;
fprintf('getting covariates is done in %f sec.\n', t2) ;


% % --------------------------------------------------------------%
%     2. Learning
% % --------------------------------------------------------------%

tic
% initialization
% tModel : specific models
%   .id : covariate type
%   .betaSet : beta Set
%   .nll : negative log-likelihood.
tModel = [] ;
for i=1:nCovSet,
    tModel(i).id = covType(i) ;
    tModel(i).betaSet = zeros(nVC, 1+dimCovSet(covType(i))) ;
    tModel(i).nll = zeros(nVC, 1) ;
    tModel(i).cif = zeros(nVC, dimCovSet(covType(i))) ;
end

% for each covariate type and each visual cluster
dInd = 0 ;
for c=1:nCovSet,
	for v=1:nVC,
	    % learning
	    [b nll] = learn_poisson_regression (full(VC(v).y_obs)', ...
    		covSet{covType(c)}, LEARNING_MODE) ;
	    tModel(c).betaSet(v,:) = b ;
	    tModel(c).nll(v) = nll ;

        % compute intensity function
        tModel(c).cif(v,:) = exp(b(1)+eye(length(b)-1)*b(2:end));
	end
end

t2 = toc ;
fprintf('learning is done in %f sec.\n', t2) ;


% % --------------------------------------------------------------%
%     3. Drawing the results
% % --------------------------------------------------------------%

hf = figure ;

% (1) draw example images of each visual cluster
for v=1:nVC,
    imgVC = draw_imgs_big_canvas(VC(v).img_example) ;
    subplot(nCovSet+2, nVC, v) ; imshow(imgVC) ;
    title(VC(v).desciption) ;
end


% (2) draw observed VC occurrences
for v=1:nVC,
    hv = subplot(nCovSet+2, nVC, nVC + v) ;
    hb = plot(1:nTick, full(VC(v).y_obs), 'r-') ;
    xlim([0 nTick+1]) ;
    title('observed VC occurrences') ;
end


% (3) draw intensity functions for year.
% get year information
year_tick = unique(fn_time_year(bins.tick)) ;
nYear = length(year_tick) ;
year_str = cell(1, nYear) ;
for i=1:nYear, year_str{i} = num2str(year_tick(i)) ; end

% draw intensity functions
for v=1:nVC,
    hv = subplot(nCovSet+2, nVC, 2*nVC + v) ;
    hb = bar(1:nYear, tModel(1).cif(v,:), 'group', 'BarWidth', 0.9, 'LineWidth', 2) ;
    set(hb,'facecolor',[0 0 1]) % or use RGB triple
    set(gca,'XTickLabel', year_str)
    xlim([0 nYear+1]) ;
    title('int. func. for year') ;
end


% (4) draw intensity functions for month.
% draw intensity functions
for v=1:nVC,
    hv = subplot(nCovSet+2, nVC, 3*nVC + v) ;
    hb = bar(1:12, tModel(2).cif(v, :), 'group', 'BarWidth', 0.9, 'LineWidth', 2) ;
    set(hb,'facecolor',[0 1 0]) % or use RGB triple
    xlim([0 13])
    title('int. func. for month') ;
end

disp('Done') ;

end


